@@ -18,12 +18,13 @@
  * The Creators of Spread are:
  *  Yair Amir, Michal Miskin-Amir, Jonathan Stanton, John Schultz.
  *
- *  Copyright (C) 1993-2013 Spread Concepts LLC <info@spreadconcepts.com>
+ *  Copyright (C) 1993-2014 Spread Concepts LLC <info@spreadconcepts.com>
  *
  *  All Rights Reserved.
  *
  * Major Contributor(s):
  * ---------------
+ *    Amy Babay            babay@cs.jhu.edu - accelerated ring protocol.
  *    Ryan Caudy           rcaudy@gmail.com - contributions to process groups.
  *    Claudiu Danilov      claudiu@acm.org - scalable wide area support.
  *    Cristina Nita-Rotaru crisn@cs.purdue.edu - group communication security.
@@ -514,7 +515,7 @@ static	char		 mess[MAX_MESSLEN];
                                                 SP_error( ret );
                                                 exit( 1 );
                                         }
-                                        for( j = 0; j < vssets[i].num_members; j++ )
+                                        for( j = 0; j < (int) vssets[i].num_members; j++ )
                                                 printf("\t%s\n", members[j] );
                                 }
 			}
